// NOTE: this filter is never directional (when distinguishing types of contours)

public class PlusContourExtractionFilter extends AveragingFilter implements ParameterizedFilter {
	private final IntFilterParam stippleRadiusParam
		= new IntFilterParam("Stipple Radius", 1, Integer.MAX_VALUE, 5);
	
	private final FilterParam[] params = new FilterParam[] {
		stippleRadiusParam
	};
	
	// ### INIT ###
	
	/**
	 * @param stippleRadius		specifies the radius of the smallest blob 
	 * 							whose contours the filter can find effectively;
	 * 							(use smaller values to detect the contours of thinner objects)
	 */
	public PlusContourExtractionFilter(int stippleRadius) {
		super(new PixelBufferFilter2[] {
			new HorizontalContourExtractionFilter(stippleRadius, false),
			new VerticalContourExtractionFilter(stippleRadius, false)
		});
	}
	
	// ### FILTER ###
	
	public void filter(int width, int height, int[] inPixels, int[] outPixels) {
		// Update parameters of base-filters
		// XXX: method of update is dependent on the order that
		//      HorizontalContourExtractionFilter and VerticalContourExtractionFilter
		//      specify their parameters in
		((IntFilterParam) ((HorizontalContourExtractionFilter) super.filters[0]).getParams()[0]).
			setValue(stippleRadiusParam.getValue());
		((IntFilterParam) ((VerticalContourExtractionFilter) super.filters[1]).getParams()[0]).
			setValue(stippleRadiusParam.getValue());
		
		// Delegate filtering to superclass
		super.filter(width, height, inPixels, outPixels);
	}
	
	public String toString() {
		return "Contour Extraction (Plus)";
	}
	
	// ### PARAMETERIZED FILTER ###
	
	public FilterParam[] getParams() {
		return params;
	}
}